splash_running() {
	$IPTABLES -n -t nat -L PREROUTING | grep -q "^splash_prerouting_all "
}

# parse one public service line
splash_public_service() {
	PROTO="${1%% *}"
	DEST="${1#* }"
	PORTS="${DEST#*:}"
	ACTION="-I"
	test "$2" = "delete" && ACTION="-D"
	test "$PROTO" = "icmp" -o "$PROTO" = "udp" -o "$PROTO" = "tcp" || unset PROTO
	test "$PROTO" = "$DEST" && unset DEST
	test "$PORTS" = "$DEST" -o "$PROTO" = "$PORTS" && unset PORTS || DEST=${DEST%%:*}
	test -n "$PROTO" && PROTO="-p $PROTO"
	test -n "$DEST" && DEST="-d $DEST"
	echo "public_services: PROTO=$PROTO, DEST=$DEST, PORTS=$PORTS, ACTION=$ACTION" >&2
	if [ -n "$PORTS" ]; then
		(IFS=",";for PR in $PORTS;do
			P1=${PR%-*}
			P2=${PR#*-}
			(IFS=" "
				$IPTABLES -t filter $ACTION splash_public_services $PROTO $DEST --dport $P1:$P2 -j RETURN || return 1
				$IPTABLES -t nat $ACTION splash_redirect $PROTO $DEST --dport $P1:$P2 -j RETURN || return 1
			)
		done)
	else
		$IPTABLES -t filter $ACTION splash_public_services $PROTO $DEST -j RETURN || return 1
		$IPTABLES -t nat $ACTION splash_redirect $PROTO $DEST -j RETURN || return 1
	fi
}

splash_update_public_services() {
	$IPTABLES -t filter -F splash_public_services

	$IPTABLES -t nat -F splash_redirect
	$IPTABLES -t nat -A splash_redirect -p tcp --dport 80 -j REDIRECT
	
	# return if we are in public mode
	if [ "$(splash_getvar ff_dhcpsplash_publicmode)" = "on" ]; then
		return 0
	fi

	# default is to drop everythin not matching a forward rule
	$IPTABLES -t filter -I splash_public_services -j DROP

	# be nice and allow DNS and icmp for everyone to anywhere
	$IPTABLES -t filter -I splash_public_services -p udp --dport 53 -j RETURN
	$IPTABLES -t filter -I splash_public_services -p icmp -j RETURN

	# allow ag/hp uris
	test "$(splash_getvar ff_dhcpsplash_ag)" = "url" && \
		aghost=$(splash_getvar ff_dhcpsplash_agurl | sed -n "s#^\(https\{0,1\}\)://\([\.0-9a-zA-Z-]*\).*#\2#p")
	test "$(splash_getvar ff_dhcpsplash_hp)" = "url" && \
		hphost=$(splash_getvar ff_dhcpsplash_hpurl | sed -n "s#^\(https\{0,1\}\)://\([\.0-9a-zA-Z-]*\).*#\2#p")
	test "$aghost" = "$hphost" && hphost=""

	for h in "$aghost" "$hphost"; do
		test -z "$h" && continue
		$IPTABLES -t filter -I splash_public_services -p tcp --dport 80 -d "$h" -j RETURN || return 1
		$IPTABLES -t nat -I splash_redirect -p tcp --dport 80 -d "$h" -j RETURN || return 1
		$IPTABLES -t filter -I splash_public_services -p tcp --dport 443 -d "$h" -j RETURN || return 1
		$IPTABLES -t nat -I splash_redirect -p tcp --dport 443 -d "$h" -j RETURN || return 1
	done
	
	# special services
	(splash_gettext publicsrv; echo) | grep -v "^#" | sed "s#[^- ,_0-9a-zA-Z:\.\#]##g" | while read line ; do
		test -z "$line" && continue
		splash_public_service "$line" || return 1
	done
}

# preauth
splash_update_preauth() {
	## read preauthenticated macs
	(splash_gettext preauth; echo) | sed "s#[^0-9a-zA-Z:]##g" |
  		while read mac hostname ; do
			test -z "$mac" && continue
			splash_isknownmac "$mac" && continue
			splash_insertmac "$mac"
  		done
}

splash_update_ipranges() {
	## flush dhcpsplash chain
	$IPTABLES -t nat -F splash_prerouting_all
	$IPTABLES -t filter -F splash_forward_all

        for iprange in $(splash_wifirange) $(splash_lanrange); do
		$IPTABLES -t nat -I splash_prerouting_all -s $iprange -j splash_prerouting
		$IPTABLES -t filter -I splash_forward_all -s $iprange -j splash_forward
        done

	$IPTABLES -t filter -I splash_forward_all -j splash_blocked
}

splash_update_blocked() {
	## remove blocked mac adresses
	$IPTABLES -t filter -F splash_blocked 
	(splash_gettext blocked; echo) | sed "s#[^0-9a-zA-Z:]##g" |
		while read mac hostname ; do
			test -z "$mac" && continue
			splash_isknownmac "$mac" && splash_removemac "$mac"
			$IPTABLES -t filter -I splash_blocked -m mac --mac-source "$mac" -j DROP >/dev/null 2>&1
		done
}

splash_update_iptables() {
	(
		splash_update_ipranges 2>/dev/null &&
		splash_update_public_services 2>/dev/null &&
		splash_update_preauth 2>/dev/null &&
		splash_update_blocked 2>/dev/null &&
		rm -f /var/run/dhcpsplash_needs_update &&
		return 0
	) || (
		$IPTABLES -t nat -F splash_prerouting_all
		$IPTABLES -t nat -F splash_prerouting
		$IPTABLES -t nat -F splash_redirect
		$IPTABLES -t filter -F splash_forward_all
		$IPTABLES -t filter -F splash_forward
		$IPTABLES -t filter -F splash_public_services
		$IPTABLES -t filter -F splash_blocked
		touch /var/run/dhcpsplash_needs_update
		return 1
	)
}
